@@ -4,7 +4,7 @@ if(!defined('POSTFIXADMIN')) {
 }
 
 /* vim: set expandtab softtabstop=4 tabstop=4 shiftwidth=4: */
-# @version $Id: upgrade.php 1798 2015-09-18 05:10:13Z gingerdog $ 
+# @version $Id: upgrade.php 1853 2016-05-22 19:58:54Z christian_boltz $ 
 
 # Note: run with upgrade.php?debug=1 to see all SQL error messages
 
@@ -72,7 +72,7 @@ function _db_field_exists($table, $field) {
     }
 }
 function _upgrade_filter_function($name) {
-    return preg_match('/upgrade_[\d]+(_mysql|_pgsql)?$/', $name) == 1; 
+    return preg_match('/upgrade_[\d]+(_mysql|_pgsql|_sqlite|_mysql_pgsql)?$/', $name) == 1; 
 }
 
 function _db_add_field($table, $field, $fieldtype, $after) {
@@ -108,8 +108,18 @@ if($CONF['database_type'] == 'pgsql') {
                     )";
         db_query_parsed($pgsql);
     }
-}
-else {
+} elseif(db_sqlite()) {
+    $enc = 'PRAGMA encoding = "UTF-8"';
+    db_query_parsed($enc);
+    $sql = "
+        CREATE TABLE {IF_NOT_EXISTS} $table (
+        `id` {AUTOINCREMENT},
+        `name` TEXT NOT NULL UNIQUE DEFAULT '',
+        `value` TEXT NOT NULL DEFAULT ''
+        )
+    ";
+    db_query_parsed($sql);
+} else {
     $mysql = "
         CREATE TABLE {IF_NOT_EXISTS} $table (
         `id` {AUTOINCREMENT} {PRIMARY},
@@ -121,21 +131,7 @@ else {
     db_query_parsed($mysql, 0, " ENGINE = MYISAM COMMENT = 'PostfixAdmin settings'");
 }
 
-$sql = "SELECT * FROM $table WHERE name = 'version'";
-
-// insert into config('version', '01');
-
-$r = db_query($sql);
-
-if($r['rows'] == 1) {
-    $rs = $r['result'];
-    $row = db_array($rs);
-    $version = $row['value'];
-} else {
-    db_query_parsed("INSERT INTO $table (name, value) VALUES ('version', '0')", 0, '');
-    $version = 0;
-}
-
+$version = check_db_version(False);
 _do_upgrade($version);
 
 function _do_upgrade($current_version) {
@@ -143,7 +139,7 @@ function _do_upgrade($current_version) {
 
     $target_version = 0;
     // Rather than being bound to an svn revision number, just look for the largest function name that matches upgrade_\d+...
-    // $target_version = preg_replace('/[^0-9]/', '', '$Revision: 1798 $');
+    // $target_version = preg_replace('/[^0-9]/', '', '$Revision: 1853 $');
     $funclist = get_defined_functions();
     $our_upgrade_functions = array_filter($funclist['user'], '_upgrade_filter_function');
     foreach($our_upgrade_functions as $function_name) {
@@ -163,21 +159,42 @@ function _do_upgrade($current_version) {
     echo "<p>Updating database:</p><p>- old version: $current_version; target version: $target_version</p>\n";
     echo "<div style='color:#999'>&nbsp;&nbsp;(If the update doesn't work, run setup.php?debug=1 to see the detailed error messages and SQL queries.)</div>";
 
+    if (db_sqlite() && $current_version < 1824) {
+        // Fast forward to the first revision supporting SQLite
+        $current_version = 1823;
+    }
+
     for ($i = $current_version +1; $i <= $target_version; $i++) {
         $function = "upgrade_$i";
+        $function_mysql_pgsql = $function . "_mysql_pgsql";
         $function_mysql = $function . "_mysql";
         $function_pgsql = $function . "_pgsql";
+        $function_sqlite = $function . "_sqlite";
+
         if (function_exists($function)) {
             echo "<p>updating to version $i (all databases)...";
             $function();
             echo " &nbsp; done";
         }
+        if ($CONF['database_type'] == 'mysql' || $CONF['database_type'] == 'mysqli' || $CONF['database_type'] == 'pgsql') {
+            if (function_exists($function_mysql_pgsql)) {
+                echo "<p>updating to version $i (MySQL and PgSQL)...";
+                $function_mysql_pgsql();
+                echo " &nbsp; done";
+            }
+        }
         if ($CONF['database_type'] == 'mysql' || $CONF['database_type'] == 'mysqli' ) {
             if (function_exists($function_mysql)) {
                 echo "<p>updating to version $i (MySQL)...";
                 $function_mysql();
                 echo " &nbsp; done";
             }
+        } elseif(db_sqlite()) {
+            if (function_exists($function_sqlite)) {
+                echo "<p>updating to version $i (SQLite)...";
+                $function_sqlite();
+                echo " &nbsp; done";
+            }
         } elseif($CONF['database_type'] == 'pgsql') {
             if (function_exists($function_pgsql)) {
                 echo "<p>updating to version $i (PgSQL)...";
@@ -220,11 +237,32 @@ function db_query_parsed($sql, $ignore_errors = 0, $attach_mysql = "") {
                 '{INNODB}'          => 'ENGINE=InnoDB',
                 '{INT}'             => 'integer NOT NULL DEFAULT 0',
                 '{BIGINT}'          => 'bigint NOT NULL DEFAULT 0',
+                '{DATETIME}'        => "datetime NOT NULL default '2000-01-01 00:00:00'", # different from {DATE} only for MySQL
                 '{DATE}'            => "timestamp NOT NULL default '2000-01-01'", # MySQL needs a sane default (no default is interpreted as CURRENT_TIMESTAMP, which is ...
                 '{DATECURRENT}'     => 'timestamp NOT NULL default CURRENT_TIMESTAMP', # only allowed once per table in MySQL
         );
         $sql = "$sql $attach_mysql";
 
+    } elseif(db_sqlite()) {
+        $replace = array(
+                '{AUTOINCREMENT}'   => 'integer PRIMARY KEY AUTOINCREMENT NOT NULL',
+                '{PRIMARY}'         => 'PRIMARY KEY',
+                '{UNSIGNED}'        => 'unsigned',
+                '{FULLTEXT}'        => 'text',
+                '{BOOLEAN}'         => "tinyint(1) NOT NULL DEFAULT '" . db_get_boolean(False) . "'",
+                '{BOOLEAN_TRUE}'    => "tinyint(1) NOT NULL DEFAULT '" . db_get_boolean(True) . "'",
+                '{UTF-8}'           => '',
+                '{LATIN1}'          => '',
+                '{IF_NOT_EXISTS}'   => 'IF NOT EXISTS',
+                '{RENAME_COLUMN}'   => 'CHANGE COLUMN',
+                '{MYISAM}'          => '',
+                '{INNODB}'          => '',
+                '{INT}'             => 'int(11) NOT NULL DEFAULT 0',
+                '{BIGINT}'          => 'bigint(20) NOT NULL DEFAULT 0',
+                '{DATETIME}'        => "datetime NOT NULL default '2000-01-01'",
+                '{DATE}'            => "datetime NOT NULL default '2000-01-01'",
+                '{DATECURRENT}'     => 'datetime NOT NULL default CURRENT_TIMESTAMP',
+        );
     } elseif($CONF['database_type'] == 'pgsql') {
         $replace = array(
                 '{AUTOINCREMENT}'   => 'SERIAL', 
@@ -244,6 +282,7 @@ function db_query_parsed($sql, $ignore_errors = 0, $attach_mysql = "") {
                 'int(10)'           => 'int', 
                 'int(11)'           => 'int', 
                 'int(4)'            => 'int', 
+                '{DATETIME}'        => "timestamp with time zone default '2000-01-01'", # stay in sync with MySQL
                 '{DATE}'            => "timestamp with time zone default '2000-01-01'", # stay in sync with MySQL
                 '{DATECURRENT}'     => 'timestamp with time zone default now()',
         );
@@ -274,7 +313,7 @@ function _drop_index ($table, $index) {
 
     if ($CONF['database_type'] == 'mysql' || $CONF['database_type'] == 'mysqli' ) {
         return "ALTER TABLE $table DROP INDEX $index";
-    } elseif($CONF['database_type'] == 'pgsql') {
+    } elseif($CONF['database_type'] == 'pgsql' || db_sqlite()) {
         return "DROP INDEX $index"; # Index names are unique with a DB for PostgreSQL
     } else {
         echo "Sorry, unsupported database type " . $conf['database_type'];
@@ -314,8 +353,8 @@ function upgrade_1_mysql() {
     CREATE TABLE {IF_NOT_EXISTS} $admin (
       `username` varchar(255) NOT NULL default '',
       `password` varchar(255) NOT NULL default '',
-      `created` datetime NOT NULL default '0000-00-00 00:00:00',
-      `modified` datetime NOT NULL default '0000-00-00 00:00:00',
+      `created` {DATETIME},
+      `modified` {DATETIME},
       `active` tinyint(1) NOT NULL default '1',
       PRIMARY KEY  (`username`)
   ) {MYISAM} COMMENT='Postfix Admin - Virtual Admins';";
@@ -325,8 +364,8 @@ function upgrade_1_mysql() {
       `address` varchar(255) NOT NULL default '',
       `goto` text NOT NULL,
       `domain` varchar(255) NOT NULL default '',
-      `created` datetime NOT NULL default '0000-00-00 00:00:00',
-      `modified` datetime NOT NULL default '0000-00-00 00:00:00',
+      `created` {DATETIME},
+      `modified` {DATETIME},
       `active` tinyint(1) NOT NULL default '1',
       PRIMARY KEY  (`address`)
     ) {MYISAM} COMMENT='Postfix Admin - Virtual Aliases'; ";
@@ -341,8 +380,8 @@ function upgrade_1_mysql() {
       `quota` bigint(20) NOT NULL default '0',
       `transport` varchar(255) default NULL,
       `backupmx` tinyint(1) NOT NULL default '0',
-      `created` datetime NOT NULL default '0000-00-00 00:00:00',
-      `modified` datetime NOT NULL default '0000-00-00 00:00:00',
+      `created` {DATETIME},
+      `modified` {DATETIME},
       `active` tinyint(1) NOT NULL default '1',
       PRIMARY KEY  (`domain`)
     ) {MYISAM} COMMENT='Postfix Admin - Virtual Domains'; ";
@@ -351,14 +390,14 @@ function upgrade_1_mysql() {
     CREATE TABLE {IF_NOT_EXISTS} $domain_admins (
       `username` varchar(255) NOT NULL default '',
       `domain` varchar(255) NOT NULL default '',
-      `created` datetime NOT NULL default '0000-00-00 00:00:00',
+      `created` {DATETIME},
       `active` tinyint(1) NOT NULL default '1',
       KEY username (`username`)
     ) {MYISAM} COMMENT='Postfix Admin - Domain Admins';";
 
     $sql[] = "
     CREATE TABLE {IF_NOT_EXISTS} $log (
-      `timestamp` datetime NOT NULL default '0000-00-00 00:00:00',
+      `timestamp` {DATETIME},
       `username` varchar(255) NOT NULL default '',
       `domain` varchar(255) NOT NULL default '',
       `action` varchar(255) NOT NULL default '',
@@ -374,8 +413,8 @@ function upgrade_1_mysql() {
       `maildir` varchar(255) NOT NULL default '',
       `quota` bigint(20) NOT NULL default '0',
       `domain` varchar(255) NOT NULL default '',
-      `created` datetime NOT NULL default '0000-00-00 00:00:00',
-      `modified` datetime NOT NULL default '0000-00-00 00:00:00',
+      `created` {DATETIME},
+      `modified` {DATETIME},
       `active` tinyint(1) NOT NULL default '1',
       PRIMARY KEY  (`username`)
     ) {MYISAM} COMMENT='Postfix Admin - Virtual Mailboxes';";
@@ -387,7 +426,7 @@ function upgrade_1_mysql() {
         body text NOT NULL, 
         cache text NOT NULL, 
         domain varchar(255) NOT NULL , 
-        created datetime NOT NULL default '0000-00-00 00:00:00', 
+        created {DATETIME},
         active tinyint(4) NOT NULL default '1', 
         PRIMARY KEY (email), 
         KEY email (email) 
@@ -546,22 +585,22 @@ function upgrade_3_mysql() {
     $table_vacation = table_by_key ('vacation');
 
     if(!_mysql_field_exists($table_admin, 'created')) {
-        db_query_parsed("ALTER TABLE $table_admin {RENAME_COLUMN} create_date created DATETIME DEFAULT '0000-00-00 00:00:00' NOT NULL;");
+        db_query_parsed("ALTER TABLE $table_admin {RENAME_COLUMN} create_date created {DATETIME};");
     }
     if(!_mysql_field_exists($table_admin, 'modified')) {
-        db_query_parsed("ALTER TABLE $table_admin {RENAME_COLUMN} change_date modified DATETIME DEFAULT '0000-00-00 00:00:00' NOT NULL;");
+        db_query_parsed("ALTER TABLE $table_admin {RENAME_COLUMN} change_date modified {DATETIME};");
     }
     if(!_mysql_field_exists($table_alias, 'created')) {
-        db_query_parsed("ALTER TABLE $table_alias {RENAME_COLUMN} create_date created DATETIME DEFAULT '0000-00-00 00:00:00' NOT NULL;");
+        db_query_parsed("ALTER TABLE $table_alias {RENAME_COLUMN} create_date created {DATETIME};");
     }
     if(!_mysql_field_exists($table_alias, 'modified')) {
-        db_query_parsed("ALTER TABLE $table_alias {RENAME_COLUMN} change_date modified DATETIME DEFAULT '0000-00-00 00:00:00' NOT NULL;");
+        db_query_parsed("ALTER TABLE $table_alias {RENAME_COLUMN} change_date modified {DATETIME};");
     }
     if(!_mysql_field_exists($table_domain, 'created')) {
-        db_query_parsed("ALTER TABLE $table_domain {RENAME_COLUMN} create_date created DATETIME DEFAULT '0000-00-00 00:00:00' NOT NULL;");
+        db_query_parsed("ALTER TABLE $table_domain {RENAME_COLUMN} create_date created {DATETIME};");
     }
     if(!_mysql_field_exists($table_domain, 'modified')) {
-        db_query_parsed("ALTER TABLE $table_domain {RENAME_COLUMN} change_date modified DATETIME DEFAULT '0000-00-00 00:00:00' NOT NULL;");
+        db_query_parsed("ALTER TABLE $table_domain {RENAME_COLUMN} change_date modified {DATETIME};");
     }
     if(!_mysql_field_exists($table_domain, 'aliases')) {
         db_query_parsed("ALTER TABLE $table_domain ADD COLUMN aliases INT(10) DEFAULT '-1' NOT NULL AFTER description;");
@@ -579,10 +618,10 @@ function upgrade_3_mysql() {
         db_query_parsed("ALTER TABLE $table_domain ADD COLUMN backupmx TINYINT(1) DEFAULT '0' NOT NULL AFTER transport;");
     }
     if(!_mysql_field_exists($table_mailbox, 'created')) {
-        db_query_parsed("ALTER TABLE $table_mailbox {RENAME_COLUMN} create_date created DATETIME DEFAULT '0000-00-00 00:00:00' NOT NULL;");
+        db_query_parsed("ALTER TABLE $table_mailbox {RENAME_COLUMN} create_date created {DATETIME};");
     }
     if(!_mysql_field_exists($table_mailbox, 'modified')) {
-        db_query_parsed("ALTER TABLE $table_mailbox {RENAME_COLUMN} change_date modified DATETIME DEFAULT '0000-00-00 00:00:00' NOT NULL;");
+        db_query_parsed("ALTER TABLE $table_mailbox {RENAME_COLUMN} change_date modified {DATETIME};");
     }
     if(!_mysql_field_exists($table_mailbox, 'quota')) {
         db_query_parsed("ALTER TABLE $table_mailbox ADD COLUMN quota INT(10) DEFAULT '-1' NOT NULL AFTER maildir;");
@@ -591,7 +630,7 @@ function upgrade_3_mysql() {
         db_query_parsed("ALTER TABLE $table_vacation ADD COLUMN domain VARCHAR(255) DEFAULT '' NOT NULL AFTER cache;");
     }
     if(!_mysql_field_exists($table_vacation, 'created')) {
-        db_query_parsed("ALTER TABLE $table_vacation ADD COLUMN created DATETIME DEFAULT '0000-00-00 00:00:00' NOT NULL AFTER domain;");
+        db_query_parsed("ALTER TABLE $table_vacation ADD COLUMN created {DATETIME} AFTER domain;");
     }
     if(!_mysql_field_exists($table_vacation, 'active')) {
         db_query_parsed("ALTER TABLE $table_vacation ADD COLUMN active TINYINT(1) DEFAULT '1' NOT NULL AFTER created;");
@@ -713,8 +752,8 @@ function upgrade_5_mysql() {
         CREATE TABLE {IF_NOT_EXISTS} `" . table_by_key('admin') . "` (
             `username` varchar(255) NOT NULL default '',
             `password` varchar(255) NOT NULL default '',
-            `created` datetime NOT NULL default '0000-00-00 00:00:00',
-            `modified` datetime NOT NULL default '0000-00-00 00:00:00',
+            `created` {DATETIME},
+            `modified` {DATETIME},
             `active` tinyint(1) NOT NULL default '1',
             PRIMARY KEY  (`username`),
     KEY username (`username`)
@@ -725,8 +764,8 @@ function upgrade_5_mysql() {
             `address` varchar(255) NOT NULL default '',
             `goto` text NOT NULL,
             `domain` varchar(255) NOT NULL default '',
-            `created` datetime NOT NULL default '0000-00-00 00:00:00',
-            `modified` datetime NOT NULL default '0000-00-00 00:00:00',
+            `created` {DATETIME},
+            `modified` {DATETIME},
             `active` tinyint(1) NOT NULL default '1',
             PRIMARY KEY  (`address`),
     KEY address (`address`)
@@ -743,8 +782,8 @@ function upgrade_5_mysql() {
             `quota` int(10) NOT NULL default '0',
             `transport` varchar(255) default NULL,
             `backupmx` tinyint(1) NOT NULL default '0',
-            `created` datetime NOT NULL default '0000-00-00 00:00:00',
-            `modified` datetime NOT NULL default '0000-00-00 00:00:00',
+            `created` {DATETIME},
+            `modified` {DATETIME},
             `active` tinyint(1) NOT NULL default '1',
             PRIMARY KEY  (`domain`),
     KEY domain (`domain`)
@@ -755,7 +794,7 @@ function upgrade_5_mysql() {
         CREATE TABLE {IF_NOT_EXISTS} `" . table_by_key('domain_admins') . "` (
             `username` varchar(255) NOT NULL default '',
             `domain` varchar(255) NOT NULL default '',
-            `created` datetime NOT NULL default '0000-00-00 00:00:00',
+            `created` {DATETIME},
             `active` tinyint(1) NOT NULL default '1',
             KEY username (`username`)
         ) {MYISAM} DEFAULT {LATIN1} COMMENT='Postfix Admin - Domain Admins';
@@ -763,7 +802,7 @@ function upgrade_5_mysql() {
 
     $result = db_query_parsed("
         CREATE TABLE {IF_NOT_EXISTS} `" . table_by_key('log') . "` (
-            `timestamp` datetime NOT NULL default '0000-00-00 00:00:00',
+            `timestamp` {DATETIME},
             `username` varchar(255) NOT NULL default '',
             `domain` varchar(255) NOT NULL default '',
             `action` varchar(255) NOT NULL default '',
@@ -780,8 +819,8 @@ function upgrade_5_mysql() {
             `maildir` varchar(255) NOT NULL default '',
             `quota` int(10) NOT NULL default '0',
             `domain` varchar(255) NOT NULL default '',
-            `created` datetime NOT NULL default '0000-00-00 00:00:00',
-            `modified` datetime NOT NULL default '0000-00-00 00:00:00',
+            `created` {DATETIME},
+            `modified` {DATETIME},
             `active` tinyint(1) NOT NULL default '1',
             PRIMARY KEY  (`username`),
     KEY username (`username`)
@@ -795,7 +834,7 @@ function upgrade_5_mysql() {
             `body` text NOT NULL,
             `cache` text NOT NULL,
             `domain` varchar(255) NOT NULL,
-            `created` datetime NOT NULL default '0000-00-00 00:00:00',
+            `created` {DATETIME},
             `active` tinyint(1) NOT NULL default '1',
             PRIMARY KEY  (`email`),
     KEY email (`email`)
@@ -837,7 +876,7 @@ function upgrade_81_mysql() { # MySQL only
 /**
  * Make logging translatable - i.e. create alias => create_alias
  */
-function upgrade_90() {
+function upgrade_90_mysql_pgsql() {
     $result = db_query_parsed("UPDATE " . table_by_key ('log') . " SET action = REPLACE(action,' ','_')", TRUE);
     # change edit_alias_state to edit_alias_active
     $result = db_query_parsed("UPDATE " . table_by_key ('log') . " SET action = 'edit_alias_state' WHERE action = 'edit_alias_active'", TRUE);
@@ -972,8 +1011,8 @@ function upgrade_438_mysql() {
         CREATE TABLE IF NOT EXISTS $table_alias_domain (
             `alias_domain` varchar(255) NOT NULL default '',
             `target_domain` varchar(255) NOT NULL default '',
-            `created` datetime NOT NULL default '0000-00-00 00:00:00',
-            `modified` datetime NOT NULL default '0000-00-00 00:00:00',
+            `created` {DATETIME},
+            `modified` {DATETIME},
             `active` tinyint(1) NOT NULL default '1',
             PRIMARY KEY  (`alias_domain`),
             KEY `active` (`active`),
@@ -1139,7 +1178,7 @@ function upgrade_504_mysql() {
     db_query_parsed("ALTER TABLE `$table_mailbox` CHANGE `local_part` `local_part` VARCHAR( 255 ) {LATIN1} NOT NULL");
 }
 
-function upgrade_655() {
+function upgrade_655_mysql_pgsql() {
     db_query_parsed(_add_index('mailbox', 'domain', 'domain'));
     db_query_parsed(_add_index('alias',   'domain', 'domain'));
 }
@@ -1206,7 +1245,7 @@ function upgrade_727_mysql() {
 }
 */
 
-function upgrade_729() {
+function upgrade_729_mysql_pgsql() {
     $table_quota = table_by_key('quota');
     $table_quota2 = table_by_key('quota2');
 
@@ -1302,11 +1341,11 @@ function upgrade_730_pgsql() {
     ");
 }
 
-function upgrade_945() {
+function upgrade_945_mysql_pgsql() {
     _db_add_field('vacation', 'modified', '{DATECURRENT}', 'created');
 }
 
-function upgrade_946() {
+function upgrade_946_mysql_pgsql() {
     # taken from upgrade_727_mysql, needs to be done for all databases
     _db_add_field('vacation', 'activefrom',  '{DATE}', 'body');
     _db_add_field('vacation', 'activeuntil', '{DATE}', 'activefrom');
@@ -1320,15 +1359,15 @@ function upgrade_968_pgsql() {
     db_query_parsed("ALTER TABLE $table_mailbox ALTER COLUMN quota    type bigint");
 }
 
-function upgrade_1050() {
+function upgrade_1050_mysql_pgsql() {
     db_query_parsed(_add_index('log', 'domain_timestamp', 'domain,timestamp'));
 }
 
-function upgrade_1283() {
+function upgrade_1283_mysql_pgsql() {
     _db_add_field('admin', 'superadmin', '{BOOLEAN}', 'password');
 }
 
-function upgrade_1284() {
+function upgrade_1284_mysql_pgsql() {
     # migrate the ALL domain to the superadmin column
     # Note: The ALL domain is not (yet) deleted to stay backwards-compatible for now (will be done in a later upgrade function)
 
@@ -1350,13 +1389,13 @@ function upgrade_1345_mysql() {
     # db_query_parsed("ALTER TABLE `$table_vacation` ADD `interval_time` INT NOT NULL DEFAULT '0' AFTER `reply_type` ");
 }
 
-function upgrade_1519() {
+function upgrade_1519_mysql_pgsql() {
     _db_add_field('fetchmail', 'sslcertck',      '{BOOLEAN}',                        'usessl'     );
     _db_add_field('fetchmail', 'sslcertpath',    "VARCHAR(255) {UTF-8}  DEFAULT ''", 'sslcertck'  );
     _db_add_field('fetchmail', 'sslfingerprint', "VARCHAR(255) {LATIN1} DEFAULT ''", 'sslcertpath');
 }
 
-function upgrade_1610() {
+function upgrade_1610_mysql_pgsql() {
     # obsoletes upgrade_1345_mysql() - which means debug mode could print "field already exists"
     _db_add_field('vacation', 'interval_time', '{INT}', 'domain');
 }
@@ -1380,7 +1419,7 @@ function upgrade_1761_mysql() {
     db_query_parsed("ALTER TABLE `$table_fetchmail`  CHANGE `date`  `date` {DATE}");
 }
 
-function upgrade_1762() {
+function upgrade_1762_mysql_pgsql() {
     _db_add_field('fetchmail', 'domain',   "VARCHAR(255) {LATIN1} DEFAULT ''", 'id');
     _db_add_field('fetchmail', 'active',   '{BOOLEAN}',                        'date');
     _db_add_field('fetchmail', 'created',  '{DATE}',                           'date');
@@ -1400,7 +1439,7 @@ function upgrade_1763_pgsql() {
     db_query_parsed("UPDATE $table SET domain=SPLIT_PART(mailbox, '@', 2) WHERE domain='';");
 }
 
-function upgrade_1767() {
+function upgrade_1767_mysql_pgsql() {
     # 'active' was just added, so make sure all existing jobs stay active
     $table = table_by_key('fetchmail');
     db_query_parsed("UPDATE $table SET active='{BOOL_TRUE}'");
@@ -1412,6 +1451,188 @@ function upgrade_1795_mysql() {
     upgrade_1761_mysql();
 }
 
+function upgrade_1824_sqlite() {
+    $admin_table = table_by_key('admin');
+    $alias_table = table_by_key('alias');
+    $alias_domain_table = table_by_key('alias_domain');
+    $domain_table = table_by_key('domain');
+    $domain_admins_table = table_by_key('domain_admins');
+    $fetchmail_table = table_by_key('fetchmail');
+    $log_table = table_by_key('log');
+    $mailbox_table = table_by_key('mailbox');
+    $quota_table = table_by_key('quota');
+    $quota2_table = table_by_key('quota2');
+    $vacation_table = table_by_key('vacation');
+    $vacation_notification_table = table_by_key('vacation_notification');
+
+    db_query_parsed("
+      CREATE TABLE $admin_table (
+          `username` varchar(255) NOT NULL,
+          `password` varchar(255) NOT NULL,
+          `superadmin` {BOOLEAN},
+          `created` {DATE},
+          `modified` {DATE},
+          `active` {BOOLEAN_TRUE},
+          {PRIMARY} (`username`));
+    ");
+
+    db_query_parsed("
+      CREATE TABLE $alias_table (
+          `address` varchar(255) NOT NULL,
+          `goto` {FULLTEXT} NOT NULL,
+          `domain` varchar(255) NOT NULL,
+          `created` {DATE},
+          `modified` {DATE},
+          `active` {BOOLEAN_TRUE},
+          {PRIMARY} (`address`));
+    ");
+
+    db_query_parsed("
+      CREATE TABLE $alias_domain_table (
+          `alias_domain` varchar(255) NOT NULL,
+          `target_domain` varchar(255) NOT NULL,
+          `created` {DATE},
+          `modified` {DATE},
+          `active` {BOOLEAN_TRUE},
+          {PRIMARY} (`alias_domain`));
+    ");
+
+    db_query_parsed("
+      CREATE TABLE $domain_table (
+          `domain` varchar(255) NOT NULL,
+          `description` varchar(255) NOT NULL,
+          `aliases` {INT},
+          `mailboxes` {INT},
+          `maxquota` {BIGINT},
+          `quota` {BIGINT},
+          `transport` varchar(255) NOT NULL,
+          `backupmx` {BOOLEAN},
+          `created` {DATE},
+          `modified` {DATE},
+          `active` {BOOLEAN_TRUE},
+          {PRIMARY} (`domain`));
+    ");
+
+    db_query_parsed("
+      CREATE TABLE $domain_admins_table (
+          `username` varchar(255) NOT NULL,
+          `domain` varchar(255) NOT NULL,
+          `created` {DATE},
+          `active` {BOOLEAN_TRUE});
+    ");
+
+    db_query_parsed("
+      CREATE TABLE $fetchmail_table (
+          `id` {AUTOINCREMENT},
+          `domain` varchar(255) DEFAULT '',
+          `mailbox` varchar(255) NOT NULL,
+          `src_server` varchar(255) NOT NULL,
+          `src_auth` varchar(255) DEFAULT NULL,
+          `src_user` varchar(255) NOT NULL,
+          `src_password` varchar(255) NOT NULL,
+          `src_folder` varchar(255) NOT NULL,
+          `poll_time` int(11)  NOT NULL DEFAULT '10',
+          `fetchall` {BOOLEAN},
+          `keep` {BOOLEAN},
+          `protocol` {FULLTEXT} DEFAULT NULL,
+          `usessl` {BOOLEAN},
+          `sslcertck` {BOOLEAN},
+          `sslcertpath` varchar(255) DEFAULT '',
+          `sslfingerprint` varchar(255) DEFAULT '',
+          `extra_options` {FULLTEXT},
+          `returned_text` {FULLTEXT},
+          `mda` varchar(255) NOT NULL,
+          `date` {DATE},
+          `created` {DATE},
+          `modified` {DATECURRENT},
+          `active` {BOOLEAN});
+    ");
+
+    db_query_parsed("
+      CREATE TABLE $log_table (
+          `timestamp` {DATE},
+          `username` varchar(255) NOT NULL,
+          `domain` varchar(255) NOT NULL,
+          `action` varchar(255) NOT NULL,
+          `data` {FULLTEXT} NOT NULL);
+    ");
+
+    db_query_parsed("
+      CREATE TABLE $mailbox_table (
+          `username` varchar(255) NOT NULL,
+          `password` varchar(255) NOT NULL,
+          `name` varchar(255) NOT NULL,
+          `maildir` varchar(255) NOT NULL,
+          `quota` {BIGINT},
+          `local_part` varchar(255) NOT NULL,
+          `domain` varchar(255) NOT NULL,
+          `created` {DATE},
+          `modified` {DATE},
+          `active` {BOOLEAN_TRUE},
+          {PRIMARY} (`username`));
+    ");
+
+    db_query_parsed("
+      CREATE TABLE $quota_table (
+          `username` varchar(255) NOT NULL,
+          `path` varchar(100) NOT NULL,
+          `current` {BIGINT},
+          {PRIMARY} (`username`,`path`));
+    ");
+
+    db_query_parsed("
+      CREATE TABLE $quota2_table (
+          `username` varchar(255) NOT NULL,
+          `bytes` {BIGINT},
+          `messages` {INT},
+          {PRIMARY} (`username`));
+    ");
+
+    db_query_parsed("
+      CREATE TABLE $vacation_table (
+          `email` varchar(255) NOT NULL,
+          `subject` varchar(255) NOT NULL,
+          `body` {FULLTEXT} NOT NULL,
+          `activefrom` {DATE},
+          `activeuntil` {DATE},
+          `cache` {FULLTEXT} NOT NULL DEFAULT '',
+          `domain` varchar(255) NOT NULL,
+          `interval_time` {INT},
+          `created` {DATE},
+          `modified` {DATECURRENT},
+          `active` {BOOLEAN_TRUE},
+          {PRIMARY} (`email`));
+    ");
+
+    db_query_parsed("
+      CREATE TABLE $vacation_notification_table (
+          `on_vacation` varchar(255) NOT NULL,
+          `notified` varchar(255) NOT NULL,
+          `notified_at` {DATECURRENT},
+          {PRIMARY} (`on_vacation`,`notified`),
+          CONSTRAINT `vacation_notification_pkey` FOREIGN KEY (`on_vacation`) REFERENCES `vacation` (`email`) ON DELETE CASCADE);
+    ");
+
+}
+
+
+function upgrade_1835_mysql() {
+    # change default values for existing datetime fields with a 0000-00-00 default to {DATETIME}
+
+    foreach (array('admin', 'alias', 'alias_domain', 'domain', 'mailbox', 'domain_admins', 'vacation') as $table_to_change) {
+        $table = table_by_key($table_to_change);
+        db_query_parsed("ALTER TABLE `$table` CHANGE `created` `created` {DATETIME}");
+    }
+
+    foreach (array('admin', 'alias', 'alias_domain', 'domain', 'mailbox') as $table_to_change) {
+        $table = table_by_key($table_to_change);
+        db_query_parsed("ALTER TABLE `$table` CHANGE `modified` `modified` {DATETIME}");
+    }
+
+    $table = table_by_key('log');
+    db_query_parsed("ALTER TABLE `$table` CHANGE `timestamp` `timestamp` {DATETIME}");
+}
+
 # TODO MySQL:
 # - various varchar fields do not have a default value
 #   https://sourceforge.net/projects/postfixadmin/forums/forum/676076/topic/3419725
